/*
 * Copyright (c) 2015-2017 elementary LLC (http://launchpad.net/elementary)
 * Copyright (c) 2017-2019 José Amuedo (https://github.com/spheras)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace DesktopFolder {
    public const string APP_ID                = "com.github.spheras.desktopfolder";
    public const string APP_DESKTOP           = APP_ID + ".desktop";
    public const string APP_TITLE             = "Desktop-Folder";
    public const string APP_NAME              = "desktopfolder";
    public const string VERSION               = "1.1.2";
    /** Name of the Folder Settings File */
    public const string FOLDER_SETTINGS_FILE  = ".desktopfolder";
    public const string OLD_NOTE_EXTENSION    = "dfn";
    public const string OLD_PHOTO_EXTENSION   = "dfp";
    public const string NEW_NOTE_EXTENSION    = "desktopnote";
    public const string NEW_PHOTO_EXTENSION   = "desktopphoto";
    /** param to show the desktop */
    public const string PARAM_SHOW_DESKTOP    = "show-desktop";
    public const string PARAM_SHOW_VERSION    = "version";
    public const int    ICON_SIZE             = 48;
    public const int    ICON_DEFAULT_WIDTH    = 90;
    // Decoration margin for windows, see application.css
    public const int WINDOW_DECORATION_MARGIN = 10;

    public const string PANEL_BLACKLIST_FILE  = ".nopanel";
    public const int    HEADERBAR_HEIGHT      = 34;

    public const int SETTINGS_VERSION         = 4;
}
